﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Runtime.CompilerServices;
using Wisej.Web;

namespace Wisej.DarkRoom
{
	public class DarkRoomPictureBox : Widget
	{

		/// <summary>
		/// Fired when the user clicks the save button.
		/// </summary>
		public event DarkRoomPictureBoxEventHandler Save;

		[Editor("Wisej.Design.ImageSourceEditor, Wisej.Framework.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=17bef35e11b84171", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
		public string ImageSource
		{
			get => this.Options.src;
			set => this.Options.src = value;
		}

		protected override void OnCreateControl()
		{
			this.Options.minWidth = this.Width;
			this.Options.minHeight = this.Height;
			this.Options.maxWidth = this.Width;
			this.Options.maxHeight = this.Height;

			base.OnCreateControl();
		}

		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string InitScript
		{
			[MethodImpl(MethodImplOptions.NoInlining)]
			get
			{
				return this.GetResourceString($"{typeof(DarkRoomPictureBox).Namespace}.DarkRoom.darkroom.js");
			}
			set { }
		}

		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override List<Package> Packages
		{
			[MethodImpl(MethodImplOptions.NoInlining)]
			get
			{
				var packages = base.Packages;
				if (packages.Count == 0)
				{
					packages.Add(new Package { 
						Name = "jquery.min.js",
						Source = "https://cdnjs.cloudflare.com/ajax/libs/jquery/2.1.3/jquery.min.js"
					});
					packages.Add(new Package
					{
						Name = "fabric.min.js",
						Source = "https://cdnjs.cloudflare.com/ajax/libs/fabric.js/1.5.0/fabric.min.js"
					});
					packages.Add(new Package
					{
						Name = "darkroom.js",
						Source = "https://rawgit.com/MattKetmo/darkroomjs/master/build/darkroom.js"
					});
					packages.Add(new Package
					{
						Name = "darkroom.css",
						Source = "https://rawgit.com/MattKetmo/darkroomjs/master/build/darkroom.css"
					});
				}
				return packages;
			}
		}

		protected override void OnWidgetEvent(WidgetEventArgs e)
		{
			if (e.Type == "save")
			{
				if (this.Save != null){
					this.Save(this, new DarkRoomPictureBoxEventArgs(ImageFromBase64(e.Data)));
				}
			}

			base.OnWidgetEvent(e);
		}

		private Image ImageFromBase64(string base64)
		{
			// data:image/png;base64,...
			if (String.IsNullOrEmpty(base64))
				return null;

			int pos = base64.IndexOf("base64,");
			if (pos < 0)
				return null;

			base64 = base64.Substring(pos + 7);
			byte[] buffer = Convert.FromBase64String(base64);
			return new Bitmap(new MemoryStream(buffer));
		}
	}
}